@@ -21,6 +21,8 @@
 
 use std::pin::Pin;
 
+use pin_project::pin_project;
+
 use crate::pin_vec::PinVec;
 
 /// Pre-allocated storage for a uniform data type, with slots of immovable
@@ -65,6 +67,7 @@ pub struct PinSlab<T> {
     next: usize,
 }
 
+#[pin_project(project = EntryProject)]
 enum Entry<T> {
     // Each slot is pre-allocated with entries of `None`.
     None,
@@ -72,7 +75,7 @@ enum Entry<T> {
     // next vacant entry.
     Vacant(usize),
     // An entry that is occupied with a value.
-    Occupied(T),
+    Occupied(#[pin] T),
 }
 
 impl<T> PinSlab<T> {
@@ -139,11 +142,12 @@ impl<T> PinSlab<T> {
 
     /// Access the given key as a pinned mutable value.
     pub fn get_pin_mut(&mut self, key: usize) -> Option<Pin<&mut T>> {
-        // Safety: all storage is pre-allocated in chunks, and each chunk
-        // doesn't move. We only provide mutators to drop the storage through
-        // `remove` (but it doesn't return it).
-        let entry = self.internal_get_mut(key)?;
-        unsafe { Some(Pin::new_unchecked(entry)) }
+        self.entries
+            .get_pin_mut(key)
+            .and_then(|entry| match entry.project() {
+                EntryProject::Occupied(entry) => Some(entry),
+                _ => None,
+            })
     }
 
     /// Get a reference to the value at the given slot.
@@ -158,7 +162,10 @@ impl<T> PinSlab<T> {
     /// assert_eq!(Some(&42), slab.get(key));
     /// ```
     pub fn get(&mut self, key: usize) -> Option<&T> {
-        self.internal_get(key)
+        self.entries.get(key).and_then(|entry| match entry {
+            Entry::Occupied(entry) => Some(entry),
+            _ => None,
+        })
     }
 
     /// Get a mutable reference to the value at the given slot.
@@ -177,27 +184,12 @@ impl<T> PinSlab<T> {
     where
         T: Unpin,
     {
-        self.internal_get_mut(key)
-    }
-
-    /// Get a mutable reference to the value at the given slot.
-    #[inline(always)]
-    fn internal_get_mut(&mut self, key: usize) -> Option<&mut T> {
         self.entries.get_mut(key).and_then(|entry| match entry {
             Entry::Occupied(entry) => Some(entry),
             _ => None,
         })
     }
 
-    /// Get a reference to the value at the given slot.
-    #[inline(always)]
-    fn internal_get(&mut self, key: usize) -> Option<&T> {
-        self.entries.get(key).and_then(|entry| match entry {
-            Entry::Occupied(entry) => Some(entry),
-            _ => None,
-        })
-    }
-
     /// Remove the key from the slab.
     ///
     /// Returns `true` if the entry was removed, `false` otherwise.
@@ -220,17 +212,18 @@ impl<T> PinSlab<T> {
     /// assert!(!slab.remove(index));
     /// ```
     pub fn remove(&mut self, key: usize) -> bool {
-        let entry = match self.entries.get_mut(key) {
+        let mut entry = match self.entries.get_pin_mut(key) {
             Some(entry) => entry,
             None => return false,
         };
 
-        match entry {
-            Entry::Occupied(..) => (),
+        match entry.as_mut().project() {
+            EntryProject::Occupied(..) => (),
             _ => return false,
         }
 
-        *entry = Entry::Vacant(self.next);
+        entry.set(Entry::Vacant(self.next));
+
         self.len -= 1;
         self.next = key;
 
@@ -259,18 +252,17 @@ impl<T> PinSlab<T> {
             self.entries
                 .extend((self.entries.len()..=key).map(|_| Entry::None));
         }
-        let entry = &mut self.entries[key];
-        self.next = match *entry {
-            Entry::None => key + 1,
-            Entry::Vacant(next) => next,
+        let mut entry = self.entries.get_pin_mut(key).unwrap();
+        self.next = match entry.as_mut().project() {
+            EntryProject::None => key + 1,
+            EntryProject::Vacant(next) => *next,
             // NB: unreachable because insert_at is an internal function,
             // which can only be appropriately called on non-occupied
             // entries. This is however, not a safety concern.
             _ => unreachable!(),
         };
 
-        *entry = Entry::Occupied(val);
-
+        entry.set(Entry::Occupied(val));
         self.len += 1;
     }
 }
@@ -280,38 +272,3 @@ impl<T> Default for PinSlab<T> {
         Self::new()
     }
 }
-
-impl<T> Drop for PinSlab<T> {
-    fn drop(&mut self) {
-        self.clear();
-    }
-}
-
-#[cfg(test)]
-mod tests {
-    use super::PinSlab;
-
-    #[global_allocator]
-    static ALLOCATOR: checkers::Allocator = checkers::Allocator::system();
-
-    #[checkers::test]
-    fn insert_get_remove_many() {
-        let mut slab = PinSlab::new();
-
-        let mut keys = Vec::new();
-
-        for i in 0..1024 {
-            keys.push((i as u128, slab.insert(Box::new(i as u128))));
-        }
-
-        for (expected, key) in keys.iter().copied() {
-            let value = slab.get_pin_mut(key).expect("value to exist");
-            assert_eq!(expected, **value.as_ref());
-            assert!(slab.remove(key));
-        }
-
-        for (_, key) in keys.iter().copied() {
-            assert!(slab.get_pin_mut(key).is_none());
-        }
-    }
-}
