@@ -10,14 +10,15 @@
 
 #![warn(clippy::undocumented_unsafe_blocks)]
 
-use crate::{lock::RwLock, pin_vec::PinVec, Shared};
-use std::{
-    cell::UnsafeCell,
-    mem::{self, ManuallyDrop},
-    ptr::{self, NonNull},
-    sync::Arc,
-    task::{Context, RawWaker, RawWakerVTable, Waker},
-};
+use std::cell::UnsafeCell;
+use std::mem::{self, ManuallyDrop};
+use std::ptr::{self, NonNull};
+use std::sync::Arc;
+use std::task::{Context, RawWaker, RawWakerVTable, Waker};
+
+use crate::lock::{Mutex, RwLock};
+use crate::pin_vec::PinVec;
+use crate::Shared;
 
 /// Wrap the current context in one that updates the local WakeSet.
 /// This takes the shared data by reference and uses `RefWaker::VTABLE`.
@@ -28,7 +29,7 @@ where
     F: FnOnce(&mut Context<'_>) -> R,
 {
     // Need to assigned owned a fixed location, so do not move it from here for the duration of the poll.
-    let internals = InternalWaker::new(NonNull::new(Arc::as_ptr(shared) as *mut _).unwrap(), index);
+    let internals = InternalWaker::new(NonNull::from(shared.as_ref()), index);
 
     // Safety: as_raw_waker() returns an object that upholds the right RawWaker contract.
     let waker = unsafe { Waker::from_raw(internals.as_waker_ref().into()) };
@@ -39,13 +40,13 @@ where
 
 /// A collection of [InternalWaker]s owned by a [Shared] task structure.
 pub(crate) struct InternalWakers {
-    wakers: parking_lot::Mutex<PinVec<InternalWaker>>,
+    wakers: Mutex<PinVec<InternalWaker>>,
 }
 
 impl InternalWakers {
     pub fn new() -> Self {
         Self {
-            wakers: parking_lot::Mutex::new(PinVec::new()),
+            wakers: Mutex::new(PinVec::new()),
         }
     }
 }
@@ -80,7 +81,7 @@ impl InternalWaker {
     }
 
     fn as_waker_ref(&self) -> InternalWakerRef {
-        InternalWakerRef(self)
+        InternalWakerRef(NonNull::from(self))
     }
 
     fn shared(&self) -> &Shared {
@@ -90,7 +91,7 @@ impl InternalWaker {
 }
 
 #[repr(transparent)]
-struct InternalWakerRef(*const InternalWaker);
+struct InternalWakerRef(NonNull<InternalWaker>);
 
 impl InternalWakerRef {
     fn get_shared_waker(shared: &Shared, index: usize) -> Self {
@@ -107,12 +108,12 @@ impl InternalWakerRef {
         unsafe {
             Arc::increment_strong_count(waker.shared.as_ptr());
         }
-        InternalWakerRef(waker)
+        InternalWakerRef(NonNull::from(waker))
     }
 
     fn internals(&self) -> &InternalWaker {
         // Safety: InternalWakerRef points to an InternalWaker by construction
-        unsafe { &*self.0 }
+        unsafe { self.0.as_ref() }
     }
 
     const VTABLE: &'static RawWakerVTable = &RawWakerVTable::new(
@@ -152,7 +153,7 @@ impl InternalWakerRef {
 
 impl From<InternalWakerRef> for RawWaker {
     fn from(val: InternalWakerRef) -> Self {
-        let waker = RawWaker::new(val.0 as *const _, InternalWakerRef::VTABLE);
+        let waker = RawWaker::new(val.0.as_ptr() as *const _, InternalWakerRef::VTABLE);
         mem::forget(val); // val will be dropped from RawWaker's vtable
         waker
     }
