@@ -1,3 +1,6 @@
+#![cfg(features = "futures-rs")]
+#![cfg(not(miri))]
+
 use futures::channel::oneshot;
 use futures::future;
 use futures::stream::StreamExt;
