@@ -1,3 +1,6 @@
+#![cfg(feature = "futures-rs")]
+#![cfg(not(miri))]
+
 use futures::future::poll_fn;
 use futures::stream::Stream;
 use std::cell::Cell;
